﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Resources;
using System.Windows.Shapes;
using System.Xml;
using System.Xml.Serialization;

namespace Controls
{
    /// <summary>
    /// Interaction logic for ItemsControls.xaml
    /// </summary>
    public partial class ItemsControls : Window
    {
        public ItemsControls()
        {
            InitializeComponent();
            var students = GetStudents();
            tcMain.DataContext = students;
            ((CollectionViewSource)FindResource("gridData")).Source = students;

        }

        private void Always(object sender, CanExecuteRoutedEventArgs e)
        {
            e.CanExecute = true;
            e.Handled = true;
        }

        private void OpenTab(object sender, ExecutedRoutedEventArgs e)
        {
            tcMain.SelectedItem = e.Parameter;
        }

        private ObservableCollection<Student> GetStudents()
        {
            var serializer = new XmlSerializer(typeof(ObservableCollection<Student>));
            var dataUri = new Uri("/Students.xml", UriKind.Relative);
            StreamResourceInfo info = Application.GetContentStream(dataUri);
            var data = serializer.Deserialize(info.Stream);
            return (ObservableCollection<Student>)data;
        }
    }

    public enum Programme
    {
        [Description("Computer Science and Information Systems")]
        CompSci,
        [Description("Mathematics")]
        Math,
        [Description("Data Science")]
        DataSci
    }

    public class Student
    {
        public String FirstName { get; set; } = string.Empty;
        public String LastName { get; set; } = string.Empty;
        [XmlIgnore]
        public Uri PhotoUri { get => new Uri(Photo); }
        [XmlIgnore]
        public Uri EmailUri { get => new Uri(Email); }
        public Programme Programme { get; set; }
        public double GradeAverage { get; set; }
        public bool IsActive { get; set; }

        public override string ToString()
        {
            return $"{FirstName} {LastName}";
        }
        public string Photo { get; set; }
        public string Email { get; set; }
    }

    public class GradeToBrushConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (targetType != typeof(Brush))
                throw new ArgumentException("Can only convert to Brush", nameof(targetType));
            decimal grade = decimal.Parse(value.ToString()??"0");
            return grade switch
            {
                _ when grade < 3.0m => Brushes.OrangeRed,
                _ when grade is >= 3.0m and < 4.0m => Brushes.Goldenrod,
                _ when grade >= 4.0m => Brushes.Green,
                _ => Brushes.Transparent
            };
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
            => DependencyProperty.UnsetValue;
    }

    public class UriToImageConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (targetType != typeof(ImageSource))
                throw new ArgumentException("Can only convert to ImageSource", nameof(targetType));
            if (value == null)
                return null;
            if (value is ImageSource)
                return new BitmapImage((Uri)value);
            return new BitmapImage(new Uri(value.ToString()));
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
