﻿using System.Windows;
using System.Windows.Controls;

namespace Precedence.Controls
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:Precedence.Controls"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:Precedence.Controls;assembly=Precedence.Controls"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Browse to and select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:MyControl/>
    ///
    /// </summary>
    public class MyControl : Control
    {
        static MyControl()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(MyControl), new FrameworkPropertyMetadata(typeof(MyControl)));
        }
        public int MyValue
        {
            get => (int)GetValue(MyValueProperty);
            set => SetValue(MyValueProperty, value);
        }

        /*
         * Whenever a dependency property allows for inheritance it should be registered as attached.
         */
        public static readonly DependencyProperty MyValueProperty =
            DependencyProperty.RegisterAttached("MyValue", typeof(int), typeof(MyControl),
                new FrameworkPropertyMetadata(0, FrameworkPropertyMetadataOptions.Inherits, null, CoerceMyValue));

        private static object CoerceMyValue(DependencyObject d, object value)
        {
            return (d as MyControl)?.MyValueOverride == true ? 12 : value;
        }

        public static int GetMyValue(DependencyObject obj)
        {
            return (int)obj.GetValue(MyValueProperty);
        }

        public static void SetMyValue(DependencyObject obj, int value)
        {
            obj.SetValue(MyValueProperty, value);
        }

        public static readonly DependencyProperty MyValueOverrideProperty =
            DependencyProperty.Register("MyValueOverride", typeof(bool), typeof(MyControl),
                new FrameworkPropertyMetadata(false,
                    (sender, value) =>
                        sender.CoerceValue(MyValueProperty)));

        public bool MyValueOverride
        {
            get => (bool)GetValue(MyValueOverrideProperty);
            set => SetValue(MyValueOverrideProperty, value);
        }
    }
}
